<?php

namespace Rtg\RtgGoogleMaps\Controller;

/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Robert Gonda <robert.gonda@gmail.com>, BSP Magnetica, s.r.o.
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Map Controller
 *
 */
class MapController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * Configuration Manager
	 *
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;

	/**
	 * Content element data
	 *
	 * @var array
	 */
	protected $cObjData;

	/**
	 * Address Repository
	 *
	 * @var \Rtg\RtgTtaddressExtend\Domain\Repository\AddressRepository
	 * @inject
	 */
	protected $addressRepository;

	/**
	 * Override these Typoscript values with non-empty Flexform values
	 *
	 * @var array
	 */
	protected $overrideDisplayTyposcriptValues = array(
		0 => 'width',
		1 => 'height',
		2 => 'zoom',
		3 => 'mapType'
	);
	protected $overrideControlsTyposcriptValues = array(
		0 => 'zoomControl',
		1 => 'mapTypeControl',
		2 => 'scaleControl',
		3 => 'streetViewControl',
		4 => 'rotateControl',
		5 => 'fullscreenControl'
	);

	/**
	 * Initialize action
	 *
	 * @return void
	 */
	public function initializeAction() {

		// Get content element data
		$this->cObjData = $this->configurationManager->getContentObject()->data;	

		// Include the JS file
		$GLOBALS['TSFE']->additionalHeaderData['rtg_google_maps'] = '<script src="'. $this->settings['googleapi']['uri'] .'" type="text/javascript"></script>';
		
		// Override Typoscript with non-empty Flexform values
		foreach($this->overrideDisplayTyposcriptValues as $name) {
			if(!empty($this->settings['flexform']['display'][$name])) {
				$this->settings['display'][$name] = $this->settings['flexform']['display'][$name];
			}
		}
		foreach($this->overrideControlsTyposcriptValues as $name) {
			$this->settings['controls'][$name] = $this->settings['flexform']['controls'][$name];
		}
	}  

    /**
     * Action show first address from active page
     *
     * @return void
     */
    public function indexAction() {

		// Get first Address from page
		$results = $this->addressRepository->findByPid($GLOBALS['TSFE']->id);
		$address = $results->getFirst();
		$this->view->assign('address', $address);

		// Assign Address location (longitude and latitude) to the view
		if($address) {
			$this->view
				->assign('latitude', (float)$address->getLatitude())
				->assign('longitude', (float)$address->getLongitude());
		}

		$this->assignMapParams();
    }

    /**
     * Action show all Addresses
     *
     * @return void
     */
    public function allAction() {

		// Get center lat/lng average from all Addresses
		$results = $this->addressRepository->findAllGeoCenter();
		$address = $results->getFirst();
		
		// Assign Address location (longitude and latitude) to the view
		if($address) {
			$this->view
				->assign('latitude', (float)$address->getLatitude())
				->assign('longitude', (float)$address->getLongitude());
		}

		$this->assignMapParams();
    }
    
    /**
     * Action show selected Addresses
     *
     * @return void
     */
    public function selectedAction() {

		// Get center lat/lng average from all Addresses
		$results = $this->addressRepository->findAllGeoCenter();
		$address = $results->getFirst();
		
		// Assign Address location (longitude and latitude) to the view
		if($address) {
			$this->view
				->assign('latitude', (float)$address->getLatitude())
				->assign('longitude', (float)$address->getLongitude());
		}

		// Get selected addresses
		if($this->settings['addresses']) {
			$addresses = $this->addressRepository->findByUids($this->settings['addresses']);
			$this->view->assign('addresses', $addresses);
		}

		/* Log
		$devlog = $this->objectManager->get('DieMedialen\\DmDeveloperlog\\Utility\\Developerlog');
		$devlog->devLog(array(
			'msg' => 'Log 1',
			'extKey' => 'rtg_goole_maps',
			'severity' => 1,
			'dataVar' => array(
				'settings' => $this->settings,
			)
		));
		*/

		$this->assignMapParams();
    }

    /**
     * Action render XML with markers (all Addresses) for map
     *
     * @return string
     */
    public function markersAction() {

		// Get all Addresses
		$addresses = $this->addressRepository->findAll();
		if($addresses) {
			// Start XML file, create parent node
			$dom = new \DOMDocument('1.0', 'UTF-8');
			$node = $dom->createElement('markers');
			$parnode = $dom->appendChild($node);
			header('Content-type: text/xml');
			foreach($addresses as $address) {
				// Add to XML document node
				$node = $dom->createElement('marker');
				$newnode = $parnode->appendChild($node);
				$newnode->setAttribute('name', $address->getName());
				$newnode->setAttribute('address', $address->getAddress().', '.$address->getCity());
				$newnode->setAttribute('lat', $address->getLatitude());
				$newnode->setAttribute('lng', $address->getLongitude());
				// [to-do]
				// $newnode->setAttribute( 'type', $address['group_uid'] );
				// $newnode->setAttribute( 'url', $this->typolink( $row['pid'] ) );
			}
			return $dom->saveXML();
		}
    }

    /**
     * Assign Maps parameters to view variables
     *
     * @return void
     */
    protected function assignMapParams() {

		// Assign view variables
		$this->view->assign('contentId', $this->cObjData['uid']);

		// Assign width and height of map
		$this->view
			->assign('width', $this->settings['display']['width'])
			->assign('height', $this->settings['display']['height']);

		// Assign icon if given by constant and/or typoscript
		if (!empty($this->settings['display']['icon']) AND file_exists(PATH_site . $this->settings['display']['icon'])) {
			$this->view->assign('icon', TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('TYPO3_REQUEST_HOST') . '/' . $this->settings['display']['icon']);
		} else {
			$this->view->assign('icon', null);
		}

		// Assign deactivation of zooming by mousewheel			
		$this->view->assign('useScrollwheel', ($this->settings['options']['useScrollwheel'] ? 'true' : 'false'));

		// Assign map zoom level to the view
		$this->view
			->assign('zoom', $this->settings['display']['zoom'])
			->assign('mapZoom', $this->settings['display']['zoom']); // back compatibility

		// Assign map type to the view
		$this->view->assign('mapType', $this->settings['display']['mapType']);

		// Assign navigation controls to the view
		$this->view->assign('mapControl', $this->settings['display']['navigationControl']);

		// Assign map styling
		$this->view
			->assign('mapStyling', NULL)
			->assign('braceStart', '{')
			->assign('braceEnd', '}');
    }
}
